package Hawat::Controller::Base;
use strict;
use warnings;
use Moose;
use namespace::autoclean;

use Scalar::Util 'blessed';
use Data::Dumper;

use Mentat::Core qw(:c_hawat);

BEGIN { extends 'Catalyst::Controller'; }

=head1 NAME

Hawat::Controller::Base - Base class for all application controllers

=head1 DESCRIPTION

Base class for all application controllers.

=head1 ACTIONS

=head2 access_denied

Handle Catalyst::Plugin::Authorization::ACL access denied exceptions

=cut

sub access_denied : Private {
    my ($self, $c) = @_;

    # User is logged in, but does not have permissions to perform requested action
    if ($c->user_exists) { $c->detach('/forbidden'); }
    # User is not logged in
    else                 { $c->detach('/unauthorized'); }
}

=head1 METHODS

=head2 flash_message
=head2 flash_info
=head2 flash_success
=head2 flash_warning
=head2 flash_error

Display various types of flash messages

=cut

sub flash_message {
    my ($self, $c, $type, $message) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);
    confess "Context must be object instance" unless blessed($c);
    push(@{$c->session->{flash_messages}->{$type}}, $message);
}
sub flash_info {
    my ($self, $c, $message) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);
    return $self->flash_message($c, 'msg_info', $message);
}
sub flash_success {
    my ($self, $c, $message) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);
    return $self->flash_message($c, 'msg_success', $message);
}
sub flash_warning {
    my ($self, $c, $message) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);
    return $self->flash_message($c, 'msg_warning', $message);
}
sub flash_error {
    my ($self, $c, $message) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);
    return $self->flash_message($c, 'msg_error', $message);
}
sub flash_debug {
    my ($self, $c, $message) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);

    if ($c->cfg('devel') and $c->check_roles('developer')) {
        return $self->flash_message($c, 'msg_debug', $message);
    }
}

=head2 dump_variable

Dump and later display given variable

=cut

sub dump_variable {
    my ($self, $c, $title, $variable, $session) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);
    confess "Context must be object instance" unless blessed($c);

    if ($c->cfg('devel') and $c->check_roles('developer')) {
        if ($session) {
            push(@{$c->stash->{dump_variables}}, {seq => $c->seq, title => $title, variable => $c->dump_variable($variable)});
        }
        else {
            push(@{$c->session->{dump_variables}}, {seq => $c->seq, title => $title, variable => $c->dump_variable($variable)});
        }
    }
}

=head2 store_query

Store and later display given database query

=cut

sub store_query {
    my ($self, $c, $title, $query, $session) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);
    confess "Context must be object instance" unless blessed($c);

    if ($c->cfg('devel') and $c->check_roles('developer')) {
        if ($session) {
            push(@{$c->stash->{dump_queries}}, {seq => $c->seq, title => $title, query => $query});
        }
        else {
            push(@{$c->session->{dump_queries}}, {seq => $c->seq, title => $title, query => $query});
        }
    }
}

=head2 name

Autodetect the component name

=cut

sub name {
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    # Determine the name of the component (last part of the class name)
    unless ($self->{NAME}) {
        my $name = blessed($self);
        my @tmp = split(/:+/, $name);
        $self->{NAME} = lc($tmp[$#tmp]);
    }
    return $self->{NAME};
}

=head2 clean_form_value

Trim spaces at the beginning and end of the form

=cut

sub clean_form_value {
    my ($self, $str) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);
    $str =~ s/^\s+//;
    $str =~ s/\s+$//;
    return $str;
}

=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

__PACKAGE__->meta->make_immutable;

1;
