package Hawat::Controller::Report;
use strict;
use warnings;
use Moose;
use namespace::autoclean;

use Data::Dumper;
use Mentat::Core qw(:c_hawat);

#-------------------------------------------------------------------------------
# Initialization section
#-------------------------------------------------------------------------------

BEGIN { extends 'Hawat::Controller::Secure'; }

__PACKAGE__->config(
    class       => 'Hawat::Controller::Report',
    constructor => 'new',
);

=head1 NAME

Hawat::Controller::Report - Catalyst controller for event reports

=head1 DESCRIPTION

Catalyst controller for handling actions related to event reports.

=head1 METHODS

=cut

#-------------------------------------------------------------------------------
# Implementation section
#-------------------------------------------------------------------------------

=head2 index

Default controller action, redirects to the 'list' action

=cut

sub index :Path :Args(0) {
    my ( $self, $c ) = @_;
    $c->response->redirect($c->uri_for($self->action_for('list')));
}

=head2 list

View the list of all reports stored in the database

=cut

sub list :Local :Args(0) {
    my ($self, $c) = @_;

    # Store the back link to this request
    $c->session->{backlink} = $c->req->uri;

    # Configure and forward to '/dbitem/find' action
    my $conf = $self->_dbicnf(
        limit  => $c->cfg('result_limit'),
        sortby => 'ts',
        sort   => 'desc',
    );
    $c->forward(HAWAT_DBI_FIND, [$conf, $c->check_roles('admin'), $c->get_memberships(), $c->cfg('reports_dir')]);

    my $fr = $c->stash->{reports};

    eval {
        $c->stash(frs => $c->model('Reports')->calculate_frs($fr));
    };
    if ($@) {
        $self->flash_error($c, "Error calculating FRS: $@");
    }

    # Choose the template to fill
    $c->stash(template => 'report/list.html.tt');
}

=head2 view

View the details of one particular report

=cut

#sub view :Local :Args(1) { # Switch to let the framework handle the error reporting
sub view :Local {
    my ($self, $c, $id) = @_;

    # Store the back link to this request
    $c->session->{backlink} = $c->req->uri;

    # Configure and forward to '/dbitem/fetch' action
    my $conf = $self->_dbicnf(
        regexp_id => '^[-_a-zA-Z0-9]+$',
    );
    $c->forward(HAWAT_DBI_FETCH, [$conf, $id, $c->cfg('reports_dir')]);

    # Restrict access to the report only for admin or members of the group
    if ($c->stash->{report_count}) {
        # Only admin can view any reports, other users may view only their own
        $c->detach('access_denied') unless ($c->check_acl({g => $c->stash->{report}->{abuse}}));

        my $os = $c->stash->{report};
        $c->stash(os => $c->model('Reports')->calculate_os($os, $c));

        # Get the list of report readers
        my ($logs, $count, $count_all, $duration, $query) = $c->model('Accesslogs')->find_report_readers(report_id => $id, limit => $c->cfg('result_limit'));
        $c->stash(report_readers => $logs);
    }

    # We must explicitly set the template, this action is called from 'Unauth'
    # controller to provide aunauth(enticated/orized) access to reports
    $c->stash(template => 'report/view.html.tt');
}

=head2 events

View the events for one particular report

=cut

#sub alerts :Local :Args(1) { # Switch to let the framework handle the error reporting
sub alerts :Local {
    my ($self, $c, $id, $alert_id) = @_;

    # HOTFIX: For some reason, without this line catalyst does not populate the $zip argument
    #$alert_id = $c->request->arguments->[1];

    # Store the back link to this request
    $c->session->{backlink} = $c->req->uri;

    # Configure and forward to '/dbitem/fetch' action
    my $conf = $self->_dbicnf(
        regexp_id => '^[-_a-zA-Z0-9]+$',
    );
    $c->forward(HAWAT_DBI_FETCH, [$conf, $id, $c->cfg('reports_dir')]);

    if ($c->stash->{report_count}) {
        my $rid = $c->stash->{report}->{id};
        my $reports_dir = $c->cfg('reports_dir');
        my $data_file_name = $c->model('Reports')->report_data_file($rid, 'json');
        my $data_file_path = $c->model('Reports')->report_data_path($reports_dir, $rid, 'json');
        if (-f $data_file_path) {
            $c->stash->{alert_id} = $alert_id;
            $c->stash->{report_alerts} = $c->model('Reports')->load_report_data_file_alerts($data_file_path);
        }
        else {
            $c->detach(HAWAT_ERROR_HANDLER, [message => "Unable to find data for report ID '$rid', data file does not exist !",
                                             message_dev => "Unable to find file $data_file_path",
                                             menu => $self->name]);
            return; # Only for readability, detach works through DIE
        }
    }

    # We must explicitly set the template, this action is called from 'Unauth'
    # controller to provide aunauth(enticated/orized) access to reports
    $c->stash(template => 'report/alerts.html.tt');
}

=head2 delete

Delete one particular report

=cut

#sub delete :Local :Args(1) {# Switch to let the framework handle the error reporting
sub delete :Local {
    my ($self, $c, $id, $confirm) = @_;

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/report');

    # Configure and forward to '/dbitem/fetch' action
    my $conf = $self->_dbicnf(
        regexp_id  => '^[-_a-zA-Z0-9]+$',
        backlink   => $backlink,
        backlink_d => $c->uri_for('/report/list'),
    );
    $c->forward(HAWAT_DBI_DELETE, [$conf, $id, $confirm, $c->cfg('reports_dir')]);
}

=head2 remail

Resend one particular report via email to appropriate abuse contact

=cut

#sub remail :Local :Args(1) {# Switch to let the framework handle the error reporting
sub remail :Local {
    my ($self, $c, $id) = @_;

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/report');

    # Determine the type of the action
    my $action = $c->request->params->{action} ||  undef;

    my $reports_dir = $c->cfg('reports_dir');

    # Perform argument sanity check: Item ID is mandatory argument
    unless ($id) {
        $c->detach(HAWAT_ERROR_HANDLER_R, [message => "Report ID was expected to be given as argument.", backlink => $backlink]);
        return; # Only for readability, detach works through DIE
    }
    # Perform argument sanity check: Regexp check on ID
    unless ($id =~ /^[-_a-zA-Z0-9]+$/) {
        $c->detach(HAWAT_ERROR_HANDLER_R, [message => "Invalid report ID '$id' received as argument.", backlink => $backlink]);
        return; # Only for readability, detach works through DIE
    }

    # Fetch the report
    my ($report, $count, $duration) = $c->model('Reports')->fetch($id, $reports_dir);
    unless ($report) {
        $c->detach(HAWAT_ERROR_HANDLER_R, [message => "Report with ID '$id' was not found in database.", backlink => $backlink]);
        return; # Only for readability, detach works through DIE
    }

    # Handle operation confirmation - display the question page
    unless ($action) {
        $c->stash(id => $id,
                  template => 'inc/dialog_confirmation_prompt.html.tt',
                  title => 'Hawat - Report remailing',
                  menu => 'reports',
                  controller => 'report',
                  action => 'remail',
                  question => "Are you really sure you want to remail report '$id'?",
                  prompt_label => 'Target email',
                  prompt_value => ($report->{mail_to} || $report->{abuse}),
                  backlink => $backlink);
        $c->forward($c->view(HAWAT_VIEW_HTML)); return;
    }
    # Handle the operation cancelation
    elsif (lc($action) =~ 'cancel') {
        $self->flash_info($c, "Remailing of report '$id' was canceled.");
        $c->response->redirect($backlink); return;
    }

    eval {
        my $target_email = $c->request->params->{prompt};
        $self->flash_info($c, "Remailing report '$id' to email '$target_email'.");
        my ($rv, $result) = $c->model('Reports')->remail($id, $reports_dir, $target_email);

        if ($rv) {
            $self->flash_success($c, "Report '$id' was successfully remailed: $result");
            $c->response->redirect($backlink); return;
        }
        else {
            $c->detach(HAWAT_ERROR_HANDLER_R, [message => "Unable to remail report '$id': $result", backlink => $backlink]);
            return; # Only for readability, detach works through DIE
        }
    };
    # Handle any exceptions, make sure never to catch 'detach' event from catalyst
    if ($@ and not $@ =~ /catalyst_detach/) {
        $c->detach(HAWAT_ERROR_HANDLER_R, [message => "$@", backlink => $backlink]);
        return; # Only for readability, detach works through DIE
    }
}

=head2 data

Get the compressed data file for one particular report

=cut

#sub data :Local :Args(1) {# Switch to let the framework handle the error reporting
sub data :Local {
    my ($self, $c, $id, $type, $zip) = @_;

    # HOTFIX: For some reason, without this line catalyst does not populate the $zip argument
    $zip = $c->request->arguments->[2];

    # Store the back link to this request
    $c->session->{backlink} = $c->req->uri;

    # Configure and forward to '/dbitem/fetch' action
    my $conf = $self->_dbicnf(
        regexp_id => '^[-_a-zA-Z0-9]+$',
    );
    $c->forward(HAWAT_DBI_FETCH, [$conf, $id, $c->cfg('reports_dir')]);

    Log::Loger->warn(__PACKAGE__, "[REPORT DATA] GET: $id, $type, $zip: ".Dumper($c->request->arguments));

    # Restrict access to the report only for admin or members of the group
    if ($c->stash->{report_count}) {
        my $rid = $c->stash->{report}->{id};
        my $reports_dir = $c->cfg('reports_dir');
        my $data_file_name = $c->model('Reports')->report_data_file($rid, $type, $zip);
        my $data_file_path = $c->model('Reports')->report_data_path($reports_dir, $rid, $type, $zip);
        if (-f $data_file_path) {
            if ($zip) {
                $c->res->header('Content-Disposition', qq[attachment; filename="$data_file_name"]);
                $c->res->content_type('application/zip');
            }
            else {
                $c->res->content_type('application/json');
            }
            $c->res->body(IO::File->new($data_file_path));
            return;
        }
        else {
            $c->detach(HAWAT_ERROR_HANDLER, [message => "Unable to find data for report ID '$rid', data file does not exist !",
                                             message_dev => "Unable to find file $data_file_path",
                                             menu => $self->name]);
            return; # Only for readability, detach works through DIE
        }
    }

    $c->detach(HAWAT_ERROR_HANDLER, [message => "Report '$id' does not exist !",
                                     menu => $self->name]);
    return; # Only for readability, detach works through DIE
}

# COMPONENT
#
#  Purpose : Perform additional instance setup after the object is created
#  Comment : Callback method, see Catalyst::Component(3) for details

#sub COMPONENT {
#   my ($class, $app, $args) = @_;
#   $args = $class->merge_config_hashes($class->config, $args);
#   #use Data::Dumper;
#   #$app->log->debug("X $class: " . Dumper($class->config));
#   #$app->log->debug("Y args  : " . Dumper($args));
#   #$app->log->debug("Z $app  : " . Dumper($app->config));
#   #$app->log->debug("MERGE XY: " . Dumper($args));
#   return $class->new($app, $args);
#}

# _dbicnf(%)
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub _dbicnf(%)
{
    my ($self, %cnf_a) = @_;
    my $cnf = {
            controller => 'Report',
            model      => HAWAT_MODEL_REPORTS(),
            items      => 'Reports',
            item       => 'Report',
        };
    foreach my $k (keys %cnf_a) {
        $cnf->{$k} = $cnf_a{$k};
    }
    return $cnf;
}

=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

__PACKAGE__->meta->make_immutable;

1;
