package Hawat::Controller::User;
use strict;
use warnings;
use Moose;
use namespace::autoclean;

use Data::Dumper;
use Mentat::Core qw(:c_hawat);

#-------------------------------------------------------------------------------
# Initialization section
#-------------------------------------------------------------------------------

BEGIN { extends 'Hawat::Controller::Secure'; }

__PACKAGE__->config(
    class       => 'Hawat::Controller::User',
    constructor => 'new',
);

=head1 NAME

Hawat::Controller::User - Catalyst controller for user management

=head1 DESCRIPTION

Catalyst controller for handling actions related to user management.

=head1 ACTIONS

=cut

#-------------------------------------------------------------------------------
# Implementation section
#-------------------------------------------------------------------------------

=head2 index

Default controller action, currently redirects to the 'list' action

=cut

sub index :Path :Args(0) {
    my ( $self, $c ) = @_;
    $c->response->redirect($c->uri_for($self->action_for('list')));
}

=head2 list

View the list of all reports stored in the database

=cut

sub list :Local :Args(0) {
    my ($self, $c) = @_;

    # Store the back link to this request
    $c->session->{backlink} = $c->req->uri;

    # Configure and forward to '/dbitem/find' action
    my $conf = $self->_dbicnf(
        limit  => $c->cfg('result_limit'),
        sortby => 'id',
        sort   => 'asc',
    );
    $c->forward(HAWAT_DBI_FIND, [$conf]);

    # Add custom data to the stash
    $c->stash(roles_all => $c->cfg('roles'));

    # TODO: make use of 'nowrap' for ajax queries
    #$c->stash(nowrap => 1);

    # Optionally dump interesting variables - dependent on the DEVEL switch
    #$self->dump_variable($c, 'Field', $variable) if $c->cfg('devel');

    # Render the view
    # NOTE: No need for that now, it will be rendered automagically, when the
    # 'end' method is called (see the Hawat::Controller::Root for details)
    #$c->forward($c->view(HAWAT_VIEW_HTML));
}

=head2 view

View the details of arbitrary user account

=cut

#sub view :Local :Args(1) { # Switch to let the framework handle the error reporting
sub view :Local {
    my ($self, $c, $id) = @_;

    # Only admin can view any accounts, other users may view only their own
    $c->detach('access_denied') unless ($c->check_acl({u => $id}));

    # Store the back link to this request
    $c->session->{backlink} = $c->req->uri;

    # Configure and forward to '/dbitem/fetch' action
    my $conf = $self->_dbicnf();
    $c->forward(HAWAT_DBI_FETCH, [$conf, $id]);

    # Add custom data to the stash
    my $users  = $c->model('Users')->list();
    my $groups = $c->model('Groups')->list();
    $c->stash(users     => $users);
    $c->stash(groups    => $groups);
    $c->stash(roles_all => $c->cfg('roles'));

    if ($c->stash->{user_count}) {
        my $managed   = $c->model('Groups')->listManagedGroups($id);
        my %managed_h = map { $_ => 1 } @$managed;
        $c->stash(managed   => $managed);
        $c->stash(managed_h => \%managed_h);

        # Get the list of report readers
        my ($logs, $count, $count_all, $duration, $query) = $c->model('Accesslogs')->find_read_reports(user_id => $id, limit => $c->cfg('result_limit'));
        $c->stash(read_reports => $logs);
    }

    # For now use the same template as the account view
    $c->stash(template => 'user/view');
}

=head2 me

View the details of account of current user

=cut

sub me :Local {
    my ($self, $c) = @_;

    # Forward to view action with current user ID
    $c->forward('view', [$c->user->id]);
}

=head2 create

Create new user account

=cut

sub create :Local {
    my ($self, $c) = @_;

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/user');

    # Configure and forward to '/dbitem/create' action
    my $conf = $self->_dbicnf(
        backlink => $backlink,
    );
    $c->forward(HAWAT_DBI_CREATE, [$conf, $c->list_roles()]);

    # Choose the common create+update template
    $c->stash(template => 'user/creatupdate');

    # Add custom data to the stash
    $c->stash(roles_all => $c->cfg('roles'));
}

=head2 update

Update existing user account

=cut

#sub update :Local :Args(1) { # Switch to let the framework handle the error reporting
sub update :Local {
    my ($self, $c, $id) = @_;

    # Only admin can view any accounts, other users may view only their own
    $c->detach('access_denied') unless ($c->check_acl({u => $id}));

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/user');

    # Configure and forward to '/dbitem/create' action
    my $conf = $self->_dbicnf(
        backlink => $backlink,
    );
    $c->forward(HAWAT_DBI_UPDATE, [$conf, $id, $c->list_roles()]);

    # Choose the common create+update template
    $c->stash(template => 'user/creatupdate');

    # Add custom data to the stash
    $c->stash(roles_all => $c->cfg('roles'));
}

=head2 rename

Rename existing user account

=cut

#sub rename :Local :Args(1) { # Switch to let the framework handle the error reporting
sub rename :Local {
    my ($self, $c, $id) = @_;

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/user');

    # Configure and forward to '/dbitem/rename' action
    my $conf = $self->_dbicnf(
        backlink => $backlink,
    );
    $c->forward(HAWAT_DBI_RENAME, [$conf, $id, $c->list_roles()]);

    # Use the general rename form
    $c->stash(template    => 'inc/dialog_rename.html.tt',
              title       => 'Mentat - Rename user account',
              rename_item => 'User');
}

=head2 delete

Delete one particular user account

=cut

#sub delete :Local :Args(1) {# Switch to let the framework handle the error reporting
sub delete :Local {
    my ($self, $c, $id, $confirm) = @_;

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/user');

    # Configure and forward to '/dbitem/delete' action
    my $conf_d = $self->_dbicnf(
        backlink   => $backlink,
        backlink_d => $c->uri_for('/user/list'),
    );
    $c->forward(HAWAT_DBI_DELETE, [$conf_d, $id, $confirm]);

    # Configure and forward to '/dbitem/modify' action
    my $conf_m = {
        controller   => 'Group',
        model        => 'Groups',
        operation    => 'deleteManagerRefs',
        item         => 'Group',
        backlink     => $c->uri_for('/user/list'),
        multi        => 1,
        msg_success  => "Successfully removed all references to user '$id' from all group records.",
        msg_none     => "There were no references to user '$id' in group records.",
        msg_failure  => "Unable to remove all references to user '$id' from all group records.",
    };
    $c->forward(HAWAT_DBI_MODIFY, [$conf_m, $id]);
}

=head2 lock

Lock user account for given user

=cut

#sub lock :Local :Args(1) {# Switch to let the framework handle the error reporting
sub lock :Local {
    my ($self, $c, $id) = @_;

    $c->forward('removeRole', [$id, 'user']);
}

=head2 unlock

Unlock user account for given user

=cut

#sub unlock :Local :Args(1) {# Switch to let the framework handle the error reporting
sub unlock :Local {
    my ($self, $c, $id) = @_;

    $c->forward('addRole', [$id, 'user']);

    # Configure and forward to '/dbitem/fetch' action
    my $conf = $self->_dbicnf();
    $c->forward(HAWAT_DBI_FETCH, [$conf, $id]);

    $c->forward('/activation_inform', [$id]);
}

=head2 addRole

Add given role to one particular user account

=cut

#sub addRole :Local :Args(2) {# Switch to let the framework handle the error reporting
sub addRole :Local {
    my ($self, $c, $id, $role) = @_;

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/user');

    # Configure and forward to '/dbitem/modify' action
    my $conf = $self->_dbicnf(
        operation => 'addRole',
        backlink  => $backlink,
    );
    if ($id and $role) {
        $conf->{msg_success} = "Successfully added role '$role' to user account '$id'.";
        $conf->{msg_failure} = "Unable to add role '$role' to user account '$id'.";
    }
    $c->forward(HAWAT_DBI_MODIFY, [$conf, $id, $role, $c->list_roles()]);
}

=head2 removeRole

Remove given role from one particular user account

=cut

#sub removeRole :Local :Args(2) {# Switch to let the framework handle the error reporting
sub removeRole :Local {
    my ($self, $c, $id, $role) = @_;

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/user');

    # Configure and forward to '/dbitem/modify' action
    my $conf = $self->_dbicnf(
        operation => 'removeRole',
        backlink  => $backlink,
    );
    if ($id and $role) {
        $conf->{msg_success} = "Successfully removed role '$role' from user account '$id'.";
        $conf->{msg_failure} = "Unable to remove role '$role' from user account '$id'.";
    }
    $c->forward(HAWAT_DBI_MODIFY, [$conf, $id, $role, $c->list_roles()]);
}

=head2 addGroup

Add given role to one particular user account

=cut

#sub addGroup :Local :Args(2) {# Switch to let the framework handle the error reporting
sub addGroup :Local {
    my ($self, $c, $id, $group) = @_;

    # Only admin can view any accounts, other users may view only their own
    $c->detach('access_denied') unless ($c->check_acl({g => $group}));

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/user');

    # Configure and forward to '/dbitem/modify' action
    my $conf = $self->_dbicnf(
        operation => 'addGroup',
        backlink  => $backlink,
    );
    if ($id and $group) {
        $conf->{msg_success} = "Successfully added user account '$id' to group '$group'.";
        $conf->{msg_failure} = "Unable to add user account '$id' to group '$group'.";
    }
    $c->forward(HAWAT_DBI_MODIFY, [$conf, $id, $group]);
}

=head2 removeGroup

Remove given group from one particular user account

=cut

#sub removeGroup :Local :Args(2) {# Switch to let the framework handle the error reporting
sub removeGroup :Local {
    my ($self, $c, $id, $group) = @_;

    # Only admin can view any accounts, other users may view only their own
    $c->detach('access_denied') unless ($c->check_acl({g => $group}));

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/user');

    # Configure and forward to '/dbitem/modify' action
    my $conf = $self->_dbicnf(
        operation => 'removeGroup',
        backlink  => $backlink,
    );
    if ($id and $group) {
        $conf->{msg_success} = "Successfully removed user account '$id' from group '$group'.";
        $conf->{msg_failure} = "Unable to remove user account '$id' from group '$group'.";
    }
    $c->forward(HAWAT_DBI_MODIFY, [$conf, $id, $group]);
}

=head2 removeQuery

Remove given stored query from one particular user account

=cut

#sub removeQuery :Local :Args(2) {# Switch to let the framework handle the error reporting
sub removeQuery :Local {
    my ($self, $c, $id, $query) = @_;

    # Only admin can view any accounts, other users may view only their own
    $c->detach('access_denied') unless ($c->check_acl({u => $id}));

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/alert');

    # Configure and forward to '/dbitem/modify' action
    my $conf = $self->_dbicnf(
        operation => 'removeQuery',
        backlink  => $backlink,
    );
    if ($id and $query) {
        $conf->{msg_success} = "Successfully removed stored query '$query' from user account '$id'.";
        $conf->{msg_failure} = "Unable to remove rstored query '$query' from user account '$id'.";
    }
    $c->forward(HAWAT_DBI_MODIFY, [$conf, $id, $query]);
}

# _dbicnf(%)
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub _dbicnf(%)
{
    my ($self, %cnf_a) = @_;
    my $cnf = {
            controller => 'User',
            model      => HAWAT_MODEL_USERS(),
            items      => 'Users',
            item       => 'User',
            menu       => 'admin',
        };
    foreach my $k (keys %cnf_a) {
        $cnf->{$k} = $cnf_a{$k};
    }
    return $cnf;
}

=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

__PACKAGE__->meta->make_immutable;

1;
