package Hawat::Model::Reports;
use strict;
use warnings;
use Moose;
use namespace::autoclean;

use Carp;
use Scalar::Util 'blessed';
use Time::HiRes qw (time);
use JSON;

use Log::Loger;
use DNS::Resolver;
use Value::Convertor;
use Mentat::Stats::Data;
use Mentat::Storage::Mongo;
use Mentat::Mailer::Report;

#-------------------------------------------------------------------------------
# Initialization section
#-------------------------------------------------------------------------------

BEGIN { extends 'Catalyst::Model::Adaptor'; }

__PACKAGE__->config(
    class           => 'Hawat::Model::Reports',
    constructor     => 'new',
    devel           => 0,
    database_name   => 'mentat',
    collection_name => 'reports_ng',
    test_mode       => 0,
);
has "devel"           => (is => 'ro', isa => 'Bool', required => 1);
has "database_name"   => (is => 'ro', isa => 'Str',  required => 1);
has "collection_name" => (is => 'ro', isa => 'Str',  required => 1);
has "mail_from"       => (is => 'ro', isa => 'Str',  required => 1);
has "mail_devel"      => (is => 'ro', isa => 'Str',  required => 1);
has "mail_admin"      => (is => 'ro', isa => 'Str',  required => 1);
has "mail_bcc"        => (is => 'ro', isa => 'Str',  required => 1);
has "mail_test"       => (is => 'ro', isa => 'Str',  required => 1);
has "reply_to"        => (is => 'ro', isa => 'Str',  required => 1);
has "test_mode"       => (is => 'ro', isa => 'Bool', required => 1);
has "reports_dir"     => (is => 'ro', isa => 'Str',  required => 1);
has "charts_dir"      => (is => 'ro', isa => 'Str',  required => 1);
has "db"              => (is => 'rw', isa => 'Mentat::Storage::Mongo');
has "stat"            => (is => 'rw', isa => 'Mentat::Stats::Data');

# Validation hashes for sorting mechanism
my %SORTBYS = (
        'id' => '_id',
        'ts' => 'ts_to',
    );
my %SORTTYPES = (
        'd' => -1,
        'a' => 1,
        'desc' => -1,
        'asc' => 1,
        'descending' => -1,
        'ascending' => 1,
    );

#-------------------------------------------------------------------------------
# Implementation section
#-------------------------------------------------------------------------------

# report_data_file($)
#
#  Usage     : my $file = $self->report_data_file($id);
#  Purpose   : Determine the name of the data file for particular report
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub report_data_file($$;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($id, $type, $zip) = @_;

    unless ($type) {
        $type = 'csv';
    }
    if ($type =~ /^json$/) {
        $type = 'json'
    }
    elsif ($type =~ /^csv$/) {
        $type = 'csv';
    }
    else {
        $type = 'json';
    }

    my $name = join('.', 'security_report_'.$id, $type);
    $name = join('.', $name, 'zip') if $zip;
    return $name;
}

# report_data_path($)
#
#  Usage     : my $path = $self->report_data_path($reports_dir, $id);
#  Purpose   : Determine path to given report
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub report_data_path($$$;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($reports_dir, $id, $type, $zip) = @_;

    return "$reports_dir/".$self->report_data_file($id, $type, $zip);
}

# load_report_data_file_alerts($)
#
#  Usage     : my $file = $self->load_report_data_file_alerts($report_file);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub load_report_data_file_alerts($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($rfile) = @_;

    my $alert_list = [];
    if (-f $rfile) {
        open (my $rf, $rfile) or die "Can`t open report file '$rfile'\n";
        my $alerts = "";
        while(my $line = <$rf>) {
            # Untaint
            if ($line =~ /^(.+)$/s) { $line = $1; }
            $alerts .= $line;
        }
        close($rf);

        eval {
            $alert_list = decode_json($alerts);
        };
    }
    return $alert_list;
}

# find($$$$$$$)
#
#  Usage     : my ($reports, $count, $count_all, $duration, $query) = $c->model('Reports')->find($search, $limit, $skip, $sortby, $sort, $admin, $memberships, $reports_dir);
#  Purpose   : Get list of all reports according to the given parameters
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub find($$$$$$$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($search, $limit, $skip, $sortby, $sort, $quick, $admin, $memberships, $reports_dir) = @_;
    die "Report directory location must be given as argument\n" unless $reports_dir;

    # SEARCH ONLY THROUGH COMPLETE REPORTS
    # Database may contain records like this: {'_id':'SOMEID'}, which work as a
    # lock for ID during report generation process. So skip those, use 'ts' as
    # marking attribute.
    # ALSO SEARCG ONLY THROUGH REPORTS WITHOUT TEST DATA
    my $query = {'ts' => {'$exists' => 1}, 'test_data' => 0};
    unless ($admin) {
        $query->{'abuse'} = {'$in' => [keys %$memberships]};
    }

    # Initialize the searching condition
    if ($search) {
        my $search_str = $search->{search};
        if ($search_str) {
            my $ip = Value::Convertor->ipv4str_to_int($search_str);

            if ($search_str =~ /^M.*/)           { $query->{'_id'}      = qr/$search_str/; }
            elsif ($ip)                          { $query->{'list_ips'} = $search_str; }
            elsif (lc($search_str) eq 'extra')   { $query->{'type'}     = 'extra'; }
            elsif (lc($search_str) eq 'summary') { $query->{'type'}     = 'summary'; }
            else {
                if ($admin) {
                    $query->{'abuse'} = qr/$search_str/;
                }
                elsif ($memberships->{$search_str}) {
                    $query->{'abuse'} = $search_str;
                }
                else {
                    die "You may not access reports for '$search_str' abuse group\n";
                }
            }
        }

        $query->{ts_from} = {'$lte' => Value::Convertor->datestr_to_unixs($search->{to})}   if $search->{to};
        $query->{ts_to}   = {'$gte' => Value::Convertor->datestr_to_unixs($search->{from})} if $search->{from};
    }

    # Initialize the sorting specification
    my $sort_spec;
    if ($sortby and $sort) {
        $sortby = lc($sortby); $sort = lc($sort);
        die "Invalid sort value '$sortby'" unless $SORTBYS{$sortby};
        die "Invalid sort type '$sort'" unless $SORTTYPES{$sort};
        $sort_spec = {$SORTBYS{$sortby} => $SORTTYPES{$sort}};
    }

    my $start = time();
    my ($reports, $count_all) = $self->db->find($query, {message => 0, attachment => 0}, $sort_spec, $limit, $skip);
    my $duration = Value::Convertor->duration_to_string(time() - $start);

    return ({}, 0, 0, $duration, $self->db->get_last_filter()) unless $count_all;

    my ($sfrom, $sto, $sts, $count);
    my $result = {};
    foreach my $r (@$reports) {
        $count++;
        $sfrom = Value::Convertor->unixs_to_ldatestr($r->{ts_from});
        $sto   = Value::Convertor->unixs_to_ldatestr($r->{ts_to});
        $sts   = Value::Convertor->unixs_to_ldatestr($r->{ts});
        $result->{$r->{ts_to}."-".$r->{ts_from}}->{t} = "$sfrom - $sto";
        $result->{$r->{ts_to}."-".$r->{ts_from}}->{tf} = "$sfrom";
        $result->{$r->{ts_to}."-".$r->{ts_from}}->{tt} = "$sto";

        $r->{id}              = $r->{_id};
        $r->{ts_from_s}       = $sfrom;
        $r->{ts_to_s}         = $sto;
        $r->{ts_s}            = $sts;
        $r->{mail_ts}         = (($r->{mail_ts})?Value::Convertor->unixs_to_ldatestr($r->{mail_ts}):'');
        $r->{attachment_file} = ((-f $self->report_data_path($reports_dir, $r->{_id})) ? 1 : 0);

        if (exists $r->{list_ips}) {
        $r->{list_ips} = [$r->{list_ips}] unless ref $r->{list_ips} eq 'ARRAY';
    }

        push(@{$result->{$r->{ts_to}."-".$r->{ts_from}}->{l}}, $r);
    }
    return ($result, $count, $count_all, $duration, $self->db->get_last_filter());
}

# fetch($$)
#
#  Usage     : my ($report, $count, $duration) = $c->model('Reports')->fetch($id, $reports_dir);
#  Purpose   : Get one particular report within the report storage
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub fetch($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($report_id, $reports_dir) = @_;
    die "Report ID must be given as argument\n" unless $report_id;
    die "Report directory location must be given as argument\n" unless $reports_dir;

    my $key = (length($report_id) == 20) ? 'ua_hash' : '_id';
    my $resolver = DNS::Resolver->new();
    my $start = time();
    my ($reports, $count) = $self->db->find({$key => $report_id, 'ts' => {'$exists' => 1}});
    my $duration = Value::Convertor->duration_to_string(time() - $start);

    return (undef, 0, $duration) unless $count;

    my $report = shift $reports;
    $report->{id}        = $report->{_id};
    $report->{downlink}  = (-f $self->report_data_path($reports_dir, $report->{_id})) ? 1 : 0;
    $report->{ts_from_s} = Value::Convertor->unixs_to_ldatestr($report->{ts_from});
    $report->{ts_to_s}   = Value::Convertor->unixs_to_ldatestr($report->{ts_to});
    $report->{ts_s}      = Value::Convertor->unixs_to_ldatestr($report->{ts});

    if (exists $report->{list_ips}) {
        my ($cnt, $ipk);
        $report->{list_ips} = [$report->{list_ips}] unless ref $report->{list_ips} eq 'ARRAY';
        foreach my $ip (@{$report->{list_ips}}) {
            next unless $ip;
            $ipk = $ip; $ipk =~ s/\./(dot)/g;
            $cnt = $report->{ips}->{$ipk};
            push(@{$report->{list_nodes}}, [$ip, $resolver->resolve_ip($ip), $cnt]);
        }
    }

    return ($report, $count, $duration);
}

# delete($$)
#
#  Usage     : my $rv = $c->model('Reports')->delete($id, $reports_dir);
#  Purpose   : Delete particular report from the report storage
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub delete($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($report_id, $reports_dir) = @_;
    die "Report ID must be given as argument\n" unless $report_id;
    die "Report directory location must be given as argument\n" unless $reports_dir;

    my $report_file = $self->report_data_path($reports_dir, $report_id);
    unlink($report_file) if -f $report_file;

    return $self->db->delete($report_id);
}

# remail($$;$)
#
#  Usage     : my ($rv, $result) = $c->model('Reports')->remail($id, $reports_dir, $target_email);
#  Purpose   : Resend particular report via email
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub remail($$;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($report_id, $reports_dir, $target_email) = @_;
    die "Report ID must be given as argument\n" unless $report_id;
    die "Report directory location must be given as argument\n" unless $reports_dir;

    my $mailer = Mentat::Mailer::Report->new(reports_dir => $reports_dir,
                                             mail_from   => $self->mail_from,
                                             mail_devel  => $self->mail_devel,
                                             mail_admin  => $self->mail_admin,
                                             mail_bcc    => $self->mail_bcc,
                                             mail_test   => $self->mail_test,
                                             reply_to    => $self->reply_to,
                                             test_mode   => $self->test_mode);
    my ($reports, $count) = $self->db->find({'_id' => $report_id, 'ts' => {'$exists' => 1}});

    return (0, "Report with ID '$report_id' was not found") unless $count;

    my $r = shift $reports;
    $r->{'to'} = $target_email || $r->{mail_to};
    my ($rc, $result, $to) = $mailer->email_report($r);

    if ($rc) {
        $r->{mail_to}  = $to;
        $r->{mail_ts}  = time;
        $r->{mail_res} = $result;
        if ($rc) { $r->{'flag_mail_sent'}++; }
        else     { $r->{'flag_mail_error'}++; }

        my $rv = $self->db->update({'_id' => $r->{_id}}, $r);

        #my $rv = { 'ok' => '1', 'n' => 1, 'connectionId' => 270, 'err' => undef };
        #SUCCESS: { 'ok' => '1', 'n' => 1, 'connectionId' => 270, 'err' => undef }
        #FAILURE: { 'ok' => '1', 'n' => 0, 'connectionId' => 271, 'err' => undef }

        unless ($rv->{ok} and $rv->{n} == 1) {
            $result .= ", however the database could not be updated";
        }
    }

    return ($rc, $result);
}

# calculate_frs($)
#
#  Usage     : my $rv = $c->model('Reports')->calculate_frs($find_result);
#  Purpose   : Calculate statistics for find result
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub calculate_frs($;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($find_result, $single) = @_;
    die "Find result must be given as argument\n" unless $find_result;

    my $r = [];
    foreach my $i (sort keys %$find_result) {
        push (@$r, @{$find_result->{$i}->{l}});
    }
    return $self->stat->ng_evaluate_reports($r, $single);
}

# calculate_os($)
#
#  Usage     : my $rv = $c->model('Reports')->calculate_os($fetch_result);
#  Purpose   : Calculate statistics for fetch result
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub calculate_os($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($fetch_result,) = @_;
    die "Fetch result must be given as argument\n" unless $fetch_result;

    return $self->stat->ng_evaluate_report($fetch_result);
}

#-------------------------------------------------------------------------------

# BUILD()
#
#  Purpose : Perform additional instance setup after the object is created
#  Comment : Callback method, see Moose::Manual::Construction(3) for details

sub BUILD {
    my $self = shift;

    # Initialize the internal database connection for whole model
    $self->db(Mentat::Storage::Mongo->new(
                    database => $self->database_name,
                    collection => $self->collection_name));
    $self->stat(Mentat::Stats::Data->new(charts_dir => $self->charts_dir));

    # If you use 'weak_ref' for moose attribute, previous call will not set the
    # attribute. It is necessary to do it with antermediate variable $self->db($r);
    die "Unable to initialize database connection attribute\n" unless $self->db;
    die "Unable to initialize report statistics attribute\n" unless $self->stat;
}

# COMPONENT
#
#  Purpose : Perform additional instance setup after the object is created
#  Comment : Callback method, see Catalyst::Component(3) for details
#
#  FIXME: For some reason config merging does not work without this code present
#         in the package!!!

sub COMPONENT {
    my ($class, $app, $args) = @_;
    $args = $class->merge_config_hashes($class->config, $args);
    #use Data::Dumper;
    #$app->log->debug("X $class: " . Dumper($class->config));
    #$app->log->debug("Y args  : " . Dumper($args));
    #$app->log->debug("Z $app  : " . Dumper($app->config));
    #$app->log->debug("MERGE XY: " . Dumper($args));
    return $class->new($app, $args);
}

1;
