package Hawat::Model::Runlogs;
use strict;
use warnings;
use Moose;
use namespace::autoclean;

use Carp;
use Scalar::Util 'blessed';
use Time::HiRes qw (time);

use Log::Loger;
use DNS::Resolver;
use Mentat::Core qw(:c_hawat);
use Value::Convertor;
use Mentat::Stats::Data;
use Mentat::Storage::Mongo;

#-------------------------------------------------------------------------------
# Initialization section
#-------------------------------------------------------------------------------

BEGIN { extends 'Catalyst::Model::Adaptor'; }

__PACKAGE__->config(
    class           => 'Hawat::Model::Runlogs',
    constructor     => 'new',
    devel           => 0,
    database_name   => 'mentat',
    collection_name => 'runlogs_reporter_ng',
    test_mode       => 0,
);
has "devel"           => (is => 'ro', isa => 'Bool', required => 1);
has "database_name"   => (is => 'ro', isa => 'Str',  required => 1);
has "collection_name" => (is => 'ro', isa => 'Str',  required => 1);
has "test_mode"       => (is => 'ro', isa => 'Bool', required => 1);
has "charts_dir"      => (is => 'ro', isa => 'Str',  required => 1);
has "db"              => (is => 'rw', isa => 'Mentat::Storage::Mongo');
has "stat"            => (is => 'rw', isa => 'Mentat::Stats::Data');

# Validation hashes for sorting mechanism
my %SORTBYS = (
        'id' => '_id',
        'ts' => 'ts_process',
    );
my %SORTTYPES = (
        'd' => -1,
        'a' => 1,
        'desc' => -1,
        'asc' => 1,
        'descending' => -1,
        'ascending' => 1,
    );

#-------------------------------------------------------------------------------
# Implementation section
#-------------------------------------------------------------------------------

# _pp($)
#
#  Usage     :
#  Purpose   : Postprocess the group object when returned from database
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub _pp($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($log,) = @_;

    $log->{id} = $log->{HAWAT_ATTR_MID()};

    return $log;
}

# find($$$$$)
#
#  Usage     : my ($runlogs, $count, $count_all, $duration, $query) = $c->model('Runlogs')->find($search, $limit, $skip, $sortby, $sort);
#  Purpose   : Get list of all reports according to the given parameters
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub find($$$$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($search, $limit, $skip, $sortby, $sort, $quick) = @_;

    # Initialize the searching condition
    my $query = {};
    if ($search) {
        my $tmp = $search->{search};
        $query->{HAWAT_ATTR_MID()} = qr/$tmp/ if $tmp;

        $query->{ts_process} = {'$lte' => Value::Convertor->datestr_to_unixs($search->{to})}   if $search->{to};
        $query->{ts_process} = {'$gte' => Value::Convertor->datestr_to_unixs($search->{from})} if $search->{from};
    }

    # Initialize the sorting specification
    my $sort_spec;
    if ($sortby and $sort) {
        $sortby = lc($sortby); $sort = lc($sort);
        die "Invalid sort value '$sortby'" unless $SORTBYS{$sortby};
        die "Invalid sort type '$sort'" unless $SORTTYPES{$sort};
        $sort_spec = {$SORTBYS{$sortby} => $SORTTYPES{$sort}};
    }

    my $start = time();
    my ($logs, $count_all) = $self->db->find($query, undef, $sort_spec, $limit, $skip);
    my $duration = Value::Convertor->duration_to_string(time() - $start);

    return ([], 0, 0, $duration, $self->db->get_last_filter()) unless $count_all;

    foreach my $l (@$logs) {
        $self->_pp($l);
    }

    return ($logs, scalar @$logs, $count_all, $duration, $self->db->get_last_filter(), $query);
}

# fetch($)
#
#  Usage     : my ($log, $count, $duration) = $c->model('Logs')->fetch($id);
#  Purpose   : Get one particular group data within the group storage
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub fetch($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($log_id,) = @_;
    die "Log ID must be given as argument\n" unless $log_id;

    my $start = time();
    my ($logs, $count) = $self->db->find({HAWAT_ATTR_MID() => MongoDB::OID->new(value => $log_id)});
    my $duration = Value::Convertor->duration_to_string(time() - $start);

    return (undef, 0, $duration) unless $count;

    my $log = shift $logs;
    $self->_pp($log);

    return ($log, $count, $duration);
}

# delete($$)
#
#  Usage     : my $rv = $c->model('Runlogs')->delete($id, $runlogs_dir);
#  Purpose   : Delete particular runlog from the runlog storage
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub delete($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($runlog_id, $runlogs_dir) = @_;
    die "Runlog ID must be given as argument\n" unless $runlog_id;
    die "Runlog directory location must be given as argument\n" unless $runlogs_dir;

    my $runlog_file = $self->runlog_data_path($runlogs_dir, $runlog_id);
    unlink($runlog_file) if -f $runlog_file;

    return $self->db->delete($runlog_id);
}

#-------------------------------------------------------------------------------

# BUILD()
#
#  Purpose : Perform additional instance setup after the object is created
#  Comment : Callback method, see Moose::Manual::Construction(3) for details

sub BUILD {
    my $self = shift;

    # Initialize the internal database connection for whole model
    $self->db(Mentat::Storage::Mongo->new(
                    database => $self->database_name,
                    collection => $self->collection_name));
    $self->stat(Mentat::Stats::Data->new(charts_dir => $self->charts_dir));

    # If you use 'weak_ref' for moose attribute, previous call will not set the
    # attribute. It is necessary to do it with antermediate variable $self->db($r);
    die "Unable to initialize database connection attribute\n" unless $self->db;
    die "Unable to initialize runlog statistics attribute\n" unless $self->stat;
}

# COMPONENT
#
#  Purpose : Perform additional instance setup after the object is created
#  Comment : Callback method, see Catalyst::Component(3) for details
#
#  FIXME: For some reason config merging does not work without this code present
#         in the package!!!

sub COMPONENT {
    my ($class, $app, $args) = @_;
    $args = $class->merge_config_hashes($class->config, $args);
    #use Data::Dumper;
    #$app->log->debug("X $class: " . Dumper($class->config));
    #$app->log->debug("Y args  : " . Dumper($args));
    #$app->log->debug("Z $app  : " . Dumper($app->config));
    #$app->log->debug("MERGE XY: " . Dumper($args));
    return $class->new($app, $args);
}

1;
