package Hawat::Model::Users;
use strict;
use warnings;
use Moose;
use namespace::autoclean;

# Fix of "smartmatch is experimental" warning
# http://blogs.perl.org/users/mike_b/2013/06/a-little-nicer-way-to-use-smartmatch-on-perl-518.html
no if $] >= 5.017011, warnings => 'experimental::smartmatch';

use Carp;
use Scalar::Util 'blessed';
use Digest::SHA qw(sha1_hex);
use HTML::Entities;

use Mentat::Core qw(:c_hawat);
use Mentat::Storage::Mongo;
use Value::Convertor;

#-------------------------------------------------------------------------------
# Initialization section
#-------------------------------------------------------------------------------

use constant USERNAME   => "username";
use constant AUTH_EDUID => "eduid";

# TODO: This is not a constant, this should be configuration, but we need some
# elegant way how to get to the configuration of another module
use constant COL_GROUPS => "groups";

#-------------------------------------------------------------------------------

BEGIN { extends 'Catalyst::Model::Adaptor'; }

__PACKAGE__->config(
    class           => 'Hawat::Model::Users',
    constructor     => 'new',
    devel           => 0,
    database_name   => 'mentat',
    collection_name => 'users',
    user_map        => {},
    default_roles   => [],
    default_groups  => [],
);
has "devel"           => (is => 'ro', isa => 'Bool',     required => 1);
has "database_name"   => (is => 'ro', isa => 'Str',      required => 1);
has "collection_name" => (is => 'ro', isa => 'Str',      required => 1);
has "user_map"        => (is => 'ro', isa => 'HashRef',  required => 1);
has "default_roles"   => (is => 'ro', isa => 'ArrayRef', required => 1);
has "default_groups"  => (is => 'ro', isa => 'ArrayRef', required => 1);
has "db"              => (is => 'rw', isa => 'Mentat::Storage::Mongo');

# Validation hashes for sorting mechanism
my %SORTBYS = (
        'id'   => HAWAT_ATTR_MID(),
        'name' => HAWAT_ATTR_NAME(),
        'org'  => HAWAT_ATTR_ORG(),
    );
my %SORTTYPES = (
        'd' => -1,
        'a' => 1,
        'desc' => -1,
        'asc' => 1,
        'descending' => -1,
        'ascending' => 1,
    );

#-------------------------------------------------------------------------------
# Implementation section
#-------------------------------------------------------------------------------

# _pp($;$)
#
#  Usage     :
#  Purpose   : Postprocess the user object when returned from database
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub _pp($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($user,) = @_;

    $user->{HAWAT_ATTR_ID()} = $user->{HAWAT_ATTR_MID()};
    map { $user->{roles_h}->{$_} = $_; } @{$user->{HAWAT_ATTR_ROLES()}};
    map { $user->{groups_h}->{$_->{id}} = $_->{id}; } @{$user->{HAWAT_ATTR_GROUPS()}};

    return $user;
}

# _unpp($)
#
#  Usage     :
#  Purpose   : Postprocess the user object when updated back into database
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub _unpp($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($user) = @_;

    delete $user->{HAWAT_ATTR_ID()};
    delete $user->{roles_h};
    delete $user->{groups_h};

    return $user;
}

# auth($)
#
#  Usage     : Called automatically via $c->authenticate() call
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub auth {
    my ($self, $c, $userinfo) = @_;

    # Fetch the user profile from database
    my $username = $userinfo->{USERNAME()};
    my ($user, $count, $duration) = $self->fetch_account($username);

    # Return the user profile data, if exist
    return $user if $user;

    # Otherwise return false
    return;
}

# register($$)
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub register($$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($userinfo, $userform, $valid_roles) = @_;

    my $user_object = {
            HAWAT_ATTR_MID()    => $userform->{'id'},
            HAWAT_ATTR_NAME()   => $userform->{'name'},
            HAWAT_ATTR_EMAIL()  => $userform->{'email'},
            HAWAT_ATTR_ORG()    => $userform->{'organization'},
            HAWAT_ATTR_TS()     => time,
            HAWAT_ATTR_ROLES()  => [],
            HAWAT_ATTR_GROUPS() => [],
        };

    my $profile = {};
    my $username = $userinfo->{id};
    $profile->{HAWAT_ATTR_MID()}   = $username;
    $profile->{HAWAT_ATTR_NAME()}  = ($userform->{name})         ? $userform->{name}         : $userinfo->{name};
    $profile->{HAWAT_ATTR_EMAIL()} = ($userform->{email})        ? $userform->{email}        : $userinfo->{email};
    $profile->{HAWAT_ATTR_ORG()}   = ($userform->{organization}) ? $userform->{organization} : $userinfo->{organization};

    $profile->{HAWAT_ATTR_TS()}        = time;
    $profile->{HAWAT_ATTR_ROLES()}     = $self->default_roles;
    $profile->{HAWAT_ATTR_GROUPS()}    = $self->default_groups;
    $profile->{HAWAT_ATTR_ORGGROUPS()} = $userinfo->{orggroups};
    $profile->{HAWAT_ATTR_AFFIL()}     = $userinfo->{affiliations};

    my @errors = $self->validate($profile, $valid_roles);
    if (@errors) {
        return ($profile, \@errors, undef);
    }

    my $rv = $self->db->insert($profile);
    return ($profile, [], $rv);
}

# find($$$$$)
#
#  Usage     : my ($users, $count, $count_all, $duration, $query) = $c->model('Users')->find($search, $limit, $skip, $sortby, $sort);
#  Purpose   : Get list of all reports according to the given parameters
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub find($$$$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($search, $limit, $skip, $sortby, $sort, $quick) = @_;

    # Initialize the searching condition
    my $query = {};
    if ($search) {
        my $tmp = $search->{search};
        $query->{'$or'} = [{HAWAT_ATTR_MID() => qr/$tmp/}, {HAWAT_ATTR_NAME() => qr/$tmp/}] if $tmp;

        $query->{ts_from} = {'$lte' => Value::Convertor->datestr_to_unixs($search->{to})}   if $search->{to};
        $query->{ts_to}   = {'$gte' => Value::Convertor->datestr_to_unixs($search->{from})} if $search->{from};
    }

    # Initialize the sorting specification
    my $sort_spec;
    if ($sortby and $sort) {
        $sortby = lc($sortby); $sort = lc($sort);
        die "Invalid sort value '$sortby'" unless $SORTBYS{$sortby};
        die "Invalid sort type '$sort'" unless $SORTTYPES{$sort};
        $sort_spec = {$SORTBYS{$sortby} => $SORTTYPES{$sort}};
    }

    my $start = time();
    my ($users, $count_all) = $self->db->find($query, undef, $sort_spec, $limit, $skip);
    my $duration = Value::Convertor->duration_to_string(time() - $start);

    return ([], 0, 0, $duration, $self->db->get_last_filter()) unless $count_all;

    foreach my $u (@$users) {
        $self->_pp($u);
    }

    return ($users, scalar @$users, $count_all, $duration, $self->db->get_last_filter());
}

# fetch($)
#
#  Usage     : my ($user, $count, $duration) = $c->model('Users')->fetch($id);
#  Purpose   : Get one particular user data within the user storage
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub fetch($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($user_id,) = @_;
    die "User ID must be given as argument\n" unless $user_id;

    my $start = time();
    my ($users, $count) = $self->db->find({HAWAT_ATTR_MID() => $user_id});
    my $duration = Value::Convertor->duration_to_string(time() - $start);

    return (undef, 0, $duration) unless $count;

    my $user = shift $users;
    $self->_pp($user);

    return ($user, $count, $duration);
}

# fetch_account($)
#
#  Usage     : my ($user, $count, $duration) = $c->model('Users')->fetch_account($id);
#  Purpose   : Get one particular user data within the user storage
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub fetch_account($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($account_id,) = @_;
    die "User account ID must be given as argument\n" unless $account_id;

    my $start = time();
    my ($users, $count) = $self->db->find({ '$or' => [
                                {HAWAT_ATTR_MID()   => $account_id},
                                {'certificate_hash' => $account_id},
                            ]});
    my $duration = Value::Convertor->duration_to_string(time() - $start);

    return (undef, 0, $duration) unless $count;

    my $user = shift $users;
    $self->_pp($user);

    return ($user, $count, $duration);
}

# list(;@)
#
#  Usage     : my $users = $c->model('Users')->list(@find_args);
#  Purpose   : Get the list of all users
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub list(;@)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    my $start = time();
    my $result = [];
    my ($users, $count) = $self->db->find(@_);
    my $duration = Value::Convertor->duration_to_string(time() - $start);

    return $result unless $count;

    map { push(@$result, $_->{HAWAT_ATTR_MID()}) } @$users;
    $result = [sort @$result];

    return $result;
}

# listGroups($)
#
#  Usage     : my ($groups, $count, $duration) = $c->model('Users')->listGroups($user_id);
#  Purpose   : Get the list of groups particular user is member of
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub listGroups($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($user_id,) = @_;
    die "User ID must be given as argument\n" unless $user_id;

    my $start = time();
    my $result = [];
    my ($users, $count) = $self->db->find({HAWAT_ATTR_MID() => $user_id});
    my $duration = Value::Convertor->duration_to_string(time() - $start);

    return $result unless $count;

    my $user = shift $users;
    map { push(@$result, $_->{HAWAT_ATTR_ID()}) } @{$user->{HAWAT_ATTR_GROUPS()}};
    $result = [sort @$result];

    return $result;
}

# listGroupMembers($)
#
#  Usage     : my ($members, $count, $duration) = $c->model('Users')->listGroupMembers($group_id);
#  Purpose   : Get the list of members of one particular group
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub listGroupMembers($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group_id,) = @_;
    die "Group ID must be given as argument\n" unless $group_id;

    my $start = time();
    my $result = [];
    my ($members, $count) = $self->db->find(
                                { HAWAT_ATTR_GROUPS().'.$id' => $group_id },
                                { HAWAT_ATTR_MID() => 1 },
                                { HAWAT_ATTR_MID() => 1 });
    my $duration = Value::Convertor->duration_to_string(time() - $start);

    return $result unless $count;

    map { push(@$result, $_->{HAWAT_ATTR_MID()}) } @$members;
    $result = [sort @$result];

    return $result;
}

# create_object($@)
#
#  Usage     : my $id = $c->model('Users')->create_object($params, $valid_roles);
#  Purpose   : Create new account within the account storage
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub create_object($@)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($form, $valid_roles) = @_;
    die "Expected user creation arguments\n" unless $form and ref $form eq 'HASH';
    die "List of valid roles must be given as argument\n" unless $valid_roles;

    my $user_object = {
            HAWAT_ATTR_MID()    => $form->{'id'},
            HAWAT_ATTR_NAME()   => $form->{'name'},
            HAWAT_ATTR_EMAIL()  => $form->{'email'},
            HAWAT_ATTR_ORG()    => $form->{'organization'},
            HAWAT_ATTR_TS()     => time,
            HAWAT_ATTR_ROLES()  => [],
            HAWAT_ATTR_GROUPS() => [],
        };

    my @errors = $self->validate($user_object, $valid_roles);
    return ($user_object, \@errors);
}

# update_object($$$)
#
#  Usage     : my $id = $c->model('Users')->update_object($user_object, $form, $valid_roles);
#  Purpose   : Create new account within the account storage
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub update_object($$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($user_object, $form, $valid_roles) = @_;
    die "User object must be given as argument\n" unless $user_object and ref $user_object eq 'HASH';
    die "Expected user form arguments\n" unless $form and ref $form eq 'HASH';
    die "List of valid roles must be given as argument\n" unless $valid_roles;

    $user_object->{HAWAT_ATTR_NAME()}  = $form->{'name'};
    $user_object->{HAWAT_ATTR_EMAIL()} = $form->{'email'};
    $user_object->{HAWAT_ATTR_ORG()}   = $form->{'organization'};

    my @errors = $self->validate($user_object, $valid_roles);
    return ($user_object, \@errors);
}

# create($@)
#
#  Usage     : my $id = $c->model('Users')->create($params, $valid_roles);
#  Purpose   : Create new account within the account storage
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub create($@)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($user_object, $valid_roles) = @_;
    die "Expected user object as argument\n" unless $user_object and ref $user_object eq 'HASH';
    die "List of valid roles must be given as argument\n" unless $valid_roles;


    my @errors = $self->validate($user_object, $valid_roles);
    return (undef, \@errors) if @errors;

    return $self->db->insert($user_object);
}

# update($$$)
#
#  Usage     : my $id = $c->model('Users')->update($id, $user_object, $valid_roles);
#  Purpose   : Create new account within the account storage
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub update($$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($user_id, $user_object, $valid_roles) = @_;
    die "User ID must be given as argument\n" unless $user_id;
    die "User object must be given as argument\n" unless $user_object and ref $user_object eq 'HASH';
    die "List of valid roles must be given as argument\n" unless $valid_roles;

    my @errors = $self->validate($user_object, $valid_roles);
    return (undef, \@errors) if @errors;

    $user_object = $self->_unpp($user_object);
    return $self->db->update({HAWAT_ATTR_MID() => $user_id}, $user_object);
}

# rename($$$$)
#
#  Usage     : my $id = $c->model('Users')->rename($id, $user_object, $new_id, $valid_roles);
#  Purpose   : Create new account within the account storage
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub rename($$$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($user_id, $user_object, $new_id, $valid_roles) = @_;
    die "User ID must be given as argument\n" unless $user_id;
    die "User object must be given as argument\n" unless $user_object and ref $user_object eq 'HASH';
    #die "Expected new user ID as argument\n" unless $new_id;
    die "List of valid roles must be given as argument\n" unless $valid_roles;

    # Replace the unique identifier
    $user_object->{HAWAT_ATTR_ID()}  = $new_id;
    $user_object->{HAWAT_ATTR_MID()} = $new_id;

    # Perform the validation to make sure, that we do not delete something too soon
    my @errors = $self->validate($user_object, $valid_roles);

    # Let user correct the mistakes
    return (undef, \@errors) if @errors;

    # Delete the previous record
    my $rv = $self->delete($user_id);
    die "Unable to delete previous record '$user_id'\n" unless ($rv->{ok} and $rv->{n});

    $user_object = $self->_unpp($user_object);

    # Reinsert the modified record
    return $self->create($user_object, $valid_roles);
}

# delete($)
#
#  Usage     : my $rv = $c->model('Users')->delete($id);
#  Purpose   : Delete particular user account from the account storage
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub delete($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($user_id,) = @_;
    die "User ID must be given as argument\n" unless $user_id;

    return $self->db->delete($user_id);
}

# addRole($$$)
#
#  Usage     : my $rv = $c->model('Users')->addRole($id, $role, $valid_roles);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub addRole($$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($user_id, $role, $valid_roles) = @_;
    die "User ID must be given as argument\n" unless $user_id;
    die "Role name must be given as argument\n" unless $role;
    die "List of valid roles must be given as argument\n" unless $valid_roles;
    die "Attempt to add invalid role '$role'\n" unless $role ~~ @$valid_roles;

    return $self->db->update(
                { HAWAT_ATTR_MID() => $user_id },
                { '$addToSet' => { HAWAT_ATTR_ROLES() => $role }}
            );
}

# removeRole($$$)
#
#  Usage     : my $rv = $c->model('Users')->removeRole($id, $role, $valid_roles);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub removeRole($$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($user_id, $role, $valid_roles) = @_;
    die "User ID must be given as argument\n" unless $user_id;
    die "Role name must be given as argument\n" unless $role;
    die "List of valid roles must be given as argument\n" unless $valid_roles;
    die "Attempt to remove invalid role '$role'\n" unless $role ~~ @$valid_roles;

    return $self->db->update(
                { HAWAT_ATTR_MID() => $user_id },
                { '$pull' => { HAWAT_ATTR_ROLES() => $role }}
            );
}

# addGroup($$)
#
#  Usage     : my $rv = $c->model('Users')->addGroup($user_id, $group_id);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub addGroup($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($user_id, $group_id) = @_;
    die "User ID must be given as argument\n" unless $user_id;
    die "Group ID must be given as argument\n" unless $group_id;

    return $self->db->update(
                { HAWAT_ATTR_MID() => $user_id },
                { '$addToSet' => {
                    HAWAT_ATTR_GROUPS() => MongoDB::DBRef->new(
                        db => $self->database_name,
                        ref => COL_GROUPS(),
                        id => $group_id)
                }}
            );
}

# removeGroup($$)
#
#  Usage     : my $rv = $c->model('Users')->removeGroup($user_id, $group_id);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub removeGroup($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($user_id, $group_id) = @_;
    die "User ID must be given as argument\n" unless $user_id;
    die "Group ID must be given as argument\n" unless $group_id;

    return $self->db->update(
                { HAWAT_ATTR_MID() => $user_id },
                { '$pull' => {
                    HAWAT_ATTR_GROUPS() => { '$id' => $group_id }
                }}
            );
}

# deleteGroupRefs($)
#
#  Usage     : my $rv = $c->model('Users')->deleteGroupRefs($group_id);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub deleteGroupRefs($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group_id,) = @_;
    die "Group ID must be given as argument\n" unless $group_id;

    return $self->db->update(
                { HAWAT_ATTR_GROUPS().'.$id' => $group_id },
                { '$pull' => {
                    HAWAT_ATTR_GROUPS() => { '$id' => $group_id }
                }},
                { 'multiple' => 1 },
            );
}

# addQuery($$)
#
#  Usage     : my $rv = $c->model('Users')->addQuery($user_id, $query_id);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub addQuery($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($user_id, $query) = @_;
    die "User ID must be given as argument\n" unless $user_id;
    die "Query must be given as argument\n" unless $query;

    return $self->db->update(
                { HAWAT_ATTR_MID() => $user_id },
                { '$addToSet' => {
                    HAWAT_ATTR_QUERIES() => $query,
                }}
            );
}

# removeQuery($$)
#
#  Usage     : my $rv = $c->model('Users')->removeQuery($user_id, $query_id);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub removeQuery($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($user_id, $query_id) = @_;
    die "User ID must be given as argument\n" unless $user_id;
    die "Query ID must be given as argument\n" unless $query_id;

    return $self->db->update(
                { HAWAT_ATTR_MID() => $user_id },
                { '$pull' => {
                    HAWAT_ATTR_QUERIES() => { HAWAT_ATTR_MID() => $query_id }
                }}
            );
}

# getQuery($$)
#
#  Usage     : my $rv = $c->model('Users')->getQuery($user_id, $query_id);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub getQuery($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($user_id, $query_id) = @_;
    die "User ID must be given as argument\n" unless $user_id;
    die "Query ID must be given as argument\n" unless $query_id;

    my ($users, $count) = $self->db->find(
            { HAWAT_ATTR_MID() => $user_id, HAWAT_ATTR_QUERIES().".".HAWAT_ATTR_MID() => $query_id },
            { HAWAT_ATTR_QUERIES() => 1 });
    return undef unless $count;

    my $user = shift $users;
    foreach my $query (@{$user->{HAWAT_ATTR_QUERIES()}}) {
        return $query if $query_id eq $query->{HAWAT_ATTR_MID()};
    }
    die "Something weird happened, the query '$query_id' should be in storage, but it isn't\n";
}

# getQueries($)
#
#  Usage     : my $rv = $c->model('Users')->getQueries($user_id);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub getQueries($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($user_id,) = @_;
    die "User ID must be given as argument\n" unless $user_id;

    my ($users, $count) = $self->db->find({ HAWAT_ATTR_MID() => $user_id });
    return undef unless $count;

    my $user = shift $users;
    my $map = {};
    map { $map->{$_->{_id}} = $_ } @{$user->{HAWAT_ATTR_QUERIES()}} if $user->{HAWAT_ATTR_QUERIES()};
    return $map;
}

# getGroupQueries($)
#
#  Usage     : my $rv = $c->model('Users')->getGroupQueries($user_id);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub getGroupQueries($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($user_id,) = @_;
    die "User ID must be given as argument\n" unless $user_id;

    my ($users, $count) = $self->db->find({ HAWAT_ATTR_MID() => $user_id });
    return undef unless $count;

    my $user = shift $users;
    my $groups = $user->{HAWAT_ATTR_GROUPS()};

    my ($group, $gid, $map);
    my $result = {};
    foreach my $g (@$groups) {
        $group = $self->db->ref_fetch($g);
        $gid   = $group->{HAWAT_ATTR_MID()};

        if ($group) {
            $map = {};
            map { $map->{$_->{_id}} = $_ } @{$group->{HAWAT_ATTR_QUERIES()}} if $group->{HAWAT_ATTR_QUERIES()};
            $result->{$gid} = $map;
        }
        else {
            die "Unable to fetch queries for group '$gid', broken reference integrity.";
        }
    }
    return $result;
}

# getDashboardQueries($)
#
#  Usage     : my $rv = $c->model('Users')->getDashboardQueries($user_id);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub getDashboardQueries($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($user_id) = @_;
    die "User ID must be given as argument\n" unless $user_id;

    my ($users, $count) = $self->db->find({ HAWAT_ATTR_MID() => $user_id });
    return undef unless $count;

    my $user = shift $users;
    my $groups = $user->{HAWAT_ATTR_GROUPS()};

    my ($group, $gid);
    my $result = {};
    foreach my $g (@$groups) {
        $group = $self->db->ref_fetch($g);
        $gid = $group->{HAWAT_ATTR_MID()};

        if ($group) {
            $result->{$gid} = $group->{HAWAT_ATTR_NETWORKS()};
        }
        else {
            die "Unable to fetch queries for group '$gid', broken reference integrity.";
        }
    }
    return $result;
}

# validate($$)
#
#  Usage     : my $id = $c->model('Users')->validate($user_object, $valid_roles);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub validate($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($user_object, $valid_roles) = @_;
    die "User object must be given as argument\n" unless $user_object and ref $user_object eq 'HASH';
    die "List of valid roles must be given as argument\n" unless $valid_roles;

    my @errors = ();

    unless ($user_object->{HAWAT_ATTR_MID()}) {
        push(@errors, ['id','Unique account identifier is mandatory']);
    }
    unless ($user_object->{HAWAT_ATTR_NAME()}) {
        push(@errors, ['name','Full name of the user is mandatory']);
    }
    unless ($user_object->{HAWAT_ATTR_ORG()}) {
        push(@errors, ['organization','Organization origin is mandatory']);
    }

    return @errors;
}

#-------------------------------------------------------------------------------

# BUILD()
#
#  Purpose : Perform additional instance setup after the object is created
#  Comment : Callback method, see Moose::Manual::Construction(3) for details

sub BUILD {
    my $self = shift;

    # Initialize the internal database connection for whole model
    $self->db(Mentat::Storage::Mongo->new(
                    database => $self->database_name,
                    collection => $self->collection_name));

    # If you use 'weak_ref' for moose attribute, previous call will not set the
    # attribute. It is necessary to do it with antermediate variable $self->db($r);
    die "Unable to initialize database connection attribute\n" unless $self->db;
}

# COMPONENT
#
#  Purpose : Perform additional instance setup after the object is created
#  Comment : Callback method, see Catalyst::Component(3) for details
#
#  FIXME: For some reason config merging does not work without this code present
#         in the package!!!

sub COMPONENT {
    my ($class, $app, $args) = @_;
    $args = $class->merge_config_hashes($class->config, $args);
    #use Data::Dumper;
    #$app->log->debug("X $class: " . Dumper($class->config));
    #$app->log->debug("Y args  : " . Dumper($args));
    #$app->log->debug("Z $app  : " . Dumper($app->config));
    #$app->log->debug("MERGE XY: " . Dumper($args));
    return $class->new($app, $args);
}

1;
